Write-Host "Script deprov_user.ps1 started..."

$Version = @{
 Path = ".\deprov_user.ps1"
 Version = "2.0"
 Author = "Sewan"
 Description = "Script for user deprovisioning"
 }
Update-ScriptFileInfo @Version -Force -ErrorAction SilentlyContinue
Test-ScriptFileInfo -Path deprov_user.ps1 -ErrorAction SilentlyContinue

#Checking the Teams PS Module
Try {
    $TeamsModuleVersion = Get-InstalledModule -Name MicrosoftTeams -RequiredVersion 2.3.1 -ErrorAction Stop
    Write-Host "Teams PS Module OK"
    } Catch { 
        Write-Warning "Your installed Teams PS module is not compatible with the script. Please install the validated version using the following command:" 
        Write-Host "Install-Module -Name MicrosoftTeams -RequiredVersion 2.3.1 -Force" -ForegroundColor Green
        Break
        }

#Start-Service winrm
Import-Module MicrosoftTeams

Function PrintUserParameters($email) {
    Get-CsOnlineUser $email | Select-Object OnPremLineURI,*Voice*,UserPrincipalName,RegistrarPool
}

Function DeprovisionUserTeamsAttribute ($email, $number, $subdomain) {
															
    # User's direct routing properties
    $CsUserProp = @{
        Identity               = $email
        EnterpriseVoiceEnabled = $false
        HostedVoicemail        = $false
        OnPremLineUri          = $null
        TelephoneNumber        = " "
    }

    # Remove TeamsOnly
    If ((Get-CsOnlineUser $CsUserProp.Identity).TeamsUpgradeEffectiveMode -eq "TeamsOnly") {
        Try {
            Grant-CsTeamsUpgradePolicy -PolicyName $null -Identity $CsUserProp.Identity -ErrorAction Stop
            Write-Host ">>> TeamsOnly coexistence mode removed from user"
        } Catch { Write-Warning ">>> Failed to remove TeamsOnly coexistence mode for the user. Continuing to next user." ; Continue }
    }

    Start-Sleep -Seconds 1

    # Removing (enterprise voice, hosted voicemail and onprem line URI)
    Try {
        Set-CsUser -identity $CsUserProp.Identity -EnterpriseVoiceEnabled $CsUserProp.EnterpriseVoiceEnabled -HostedVoicemail $CsUserProp.HostedVoicemail -OnPremlineURI $CsUserProp.OnPremLineURI -ErrorAction Stop
        Write-Host ">>> User parameters removed (enterprise voice, hosted voicemail, onprem line URI)"
    } Catch { Write-Warning ">>> Failed to remove user parameters" }

    Start-Sleep -Seconds 1

    # Removing the routing policy
    Try {
        Grant-CsOnlineVoiceRoutingPolicy -Identity $CsUserProp.Identity -PolicyName $Null -ErrorAction Stop
        Write-Host ">>> Routing policy has been removed for this user"
    } Catch { Write-Warning ">>> Failed to remove routing policy from user" }

    Start-Sleep -Seconds 1

   if ($remove_numdir -eq 0)
   {
   # Remove phone number from Azure AD 
    Try {
        Set-AzureADUser -ObjectId $CsUserProp.Identity -TelephoneNumber $CsUserProp.TelephoneNumber -ErrorAction Stop
        Write-Host ">>> User phone number has been removed from directory"
    } Catch { Write-Host ">>> Failed to remove user phone number from directory" -ForegroundColor Red }

    Start-Sleep -Seconds 1
   }

}

# Prompt connection method
$title    = 'Authentication'
$question = "Are you using MFA ?"
$choices  = '&Yes', '&No'
$conn_method = $Host.UI.PromptForChoice($title, $question, $choices, 1)
							  																																	
# Prompt remove phone number from Azure AD
$title    = 'Azure AD user phone number'
$question = "Do you want to remove user's phone number from Azure AD ?"
$choices  = '&Yes', '&No'
$remove_numdir = $Host.UI.PromptForChoice($title, $question, $choices, 1)

if ($remove_numdir -eq 0)
{
   # Checking the AzureAD PS Module
   Try {
        $AzureModuleVersion = Get-InstalledModule -Name AzureAD -RequiredVersion 2.0.2.135 -ErrorAction Stop
        Write-Host "AzureAD PS module OK"
    } Catch { 
        Write-Warning "Your installed AzureAD PS module is not compatible with the script. Please install the validated version using the following command:" 
        Write-Host "Install-Module -Name AzureAD -RequiredVersion 2.0.2.135 -Force" -ForegroundColor Green
        Break
    }
}

if ($conn_method -eq 1)
{
    if (Test-Path ./global_vars.ps1 -PathType Leaf)
    {
        Write-Host "Global variables file found"
        . ./global_vars.ps1
        $passwd = convertto-securestring -AsPlainText -Force -String $password
    }
    else 
    {
        Write-Warning "Global variables global_vars.ps1 not found"
        Write-Host "Please fill in the following fields:"

        $adminaccount = Read-Host -Prompt "Admin login"
        $passwd = Read-Host -Prompt "Admin password" -AsSecureString
    }
}

$use_csv = $False
if (! (Test-Path ./deprov_teamsusers.csv -PathType Leaf))
{
    Write-Host "deprov_teamsusers.csv missing, please fill in the following fields:"
    $PhoneNumber = Read-Host -Prompt "User PhoneNumber (E.164)"
    $UserUPN = Read-Host -Prompt "User UPN (email)"

} 
else { $use_csv = $True }


if ($conn_method -eq 1) 
{
# Connection part (not using MFA)
$cred = New-Object System.Management.Automation.PSCredential ($adminaccount, $passwd)
$sfboSession = Connect-MicrosoftTeams -Credential $cred
Write-Host "Connected to Teams"
Start-Sleep -Milliseconds 500

    if ($remove_numdir -eq 0)
    { 
    $sfboSessionazure = Connect-AzureAD -Credential $cred
    Write-Host "Connected to Azure AD"
    Start-Sleep -Milliseconds 500
    }    
}
else {

# Connection part (using MFA)
$sfboSession = Connect-MicrosoftTeams
Write-Host "Connected to Teams"
Start-Sleep -Milliseconds 500

    if ($remove_numdir -eq 0)
    {
    $sfboSessionazure = Connect-AzureAD
    Write-Host "Connected to Azure AD"
    Start-Sleep -Milliseconds 500
    }
}

if ($use_csv)
{
    ForEach ($u in (Import-Csv ./deprov_teamsusers.csv)) 
    {
        $UserUPN = $u.UserUPN
        $PhoneNumber = $u.PhoneNumber

        Write-host "> $UserUPN"
        Write-host "Before:"
        PrintUserParameters $UserUPN
        DeprovisionUserTeamsAttribute $UserUPN $PhoneNumber $subdomain
        Write-host "After:"
        PrintUserParameters $UserUPN
        Write-host "> $UserUPN done" -ForegroundColor Green
    }
}
else 
{
    Write-host "> $UserUPN"
    Write-host "Before:"
    PrintUserParameters $UserUPN
    DeprovisionUserTeamsAttribute $UserUPN $PhoneNumber $subdomain
    Write-host "After:"
    PrintUserParameters $UserUPN
    Write-host "> $UserUPN done" -ForegroundColor Green
}

if ($remove_numdir -eq 0) {
Disconnect-AzureAD
}

Disconnect-MicrosoftTeams
Exit
