Write-Host "Script domain.ps1 started..."

$Version = @{
 Path = ".\domain.ps1"
 Version = "2.0"
 Author = "Sewan"
 Description = "Script for domain provisioning"
 }
Update-ScriptFileInfo @Version -Force -ErrorAction SilentlyContinue
Test-ScriptFileInfo -Path domain.ps1 -ErrorAction SilentlyContinue

#Checking the Teams PS Module
Try {
    $TeamsModuleVersion = Get-InstalledModule -Name MicrosoftTeams -RequiredVersion 2.3.1 -ErrorAction Stop
    Write-Host "Teams PS Module OK"
    } Catch { 
        Write-Warning "Your installed Teams PS module is not compatible with the script. Please install the validated version using the following command:" 
        Write-Host "Install-Module -Name MicrosoftTeams -RequiredVersion 2.3.1 -Force" -ForegroundColor Green
        Break
        }

#Start-Service winrm
Import-Module MicrosoftTeams

# Prompt connection method
$title    = 'Authentication'
$question = "Are you using MFA ?"
$choices  = '&Yes', '&No'
$conn_method = $Host.UI.PromptForChoice($title, $question, $choices, 1)

if ($conn_method -eq 1)
{
	if (Test-Path ./global_vars.ps1 -PathType Leaf)
	{
		Write-Host "Global variables file found"
		. ./global_vars.ps1
		$passwd = convertto-securestring -AsPlainText -Force -String $password
	}
	else 
	{
		Write-Warning "Global variables global_vars.ps1 not found"
		Write-Host "Please fill in the following fields:"

		$adminaccount = Read-Host -Prompt "Admin login"
		$passwd = Read-Host -Prompt "Admin password" -AsSecureString
		$activationuser = Read-Host -Prompt "Acivation account login"
	}
}
else
{
[void][System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic')
$activationuser = [Microsoft.VisualBasic.Interaction]::InputBox("Domain activation account login (ex: domaineactivation@XXXXXX.teamssoftphone.com)", "Please fill in")
}

$subdomain = $activationuser.Split("{@}").Get(1)

Write-Host "Connection to Microsoft Online using:"
Write-Host "  Admin login: $adminaccount"
Write-Host "  Activation account: $activationuser"
Write-Host "  Subdomain: $subdomain"

if ($conn_method -eq 1)
{
# Connection part (not using MFA)
$cred = New-Object System.Management.Automation.PSCredential ($adminaccount, $passwd)
$sfboSession = Connect-MicrosoftTeams -Credential $cred
}
else {
# Connection part (using MFA)
$sfboSession = Connect-MicrosoftTeams
}
Write-Host "Connected to Teams"
Start-Sleep -Milliseconds 500


####### ! PRELIMINARY CHECKS, IF UNSUCCESSFULL, YOU HAVE TO CHECK THAT $activationuser EXISTS WITH THE SUBDOMAIN SUFFIX AND PREREQUSITES LICENCES (E3 + PHONE OR E5) ####### 
####### ! THIS PART CAN NEED UP TO 24H  ####### 

# Check if activation account is validated, if not empty you can continue, else you can wait up to 24 hours
Write-Host "1. Checking $activationuser"
$csonlineuser = Get-CsOnlineUser -Identity $activationuser
if (! $csonlineuser.RegistrarPool) 
{
    Write-Host "Problem found. Please verify $activationuser before continuing." -ForegroundColor Yellow
    Disconnect-MicrosoftTeams
    Exit
}
Write-Host "   Account $activationuser is OK"

Start-Sleep -Seconds 1


### Step 2: PSTN USAGE

Write-Host "2. CsOnlinePstnUsage rule"
$hasonlinepstnusage = (Get-CsOnlinePstnUsage -Filter Global).Usage -contains "$subdomain"
if ($hasonlinepstnusage)
{
    Write-Host "   CsOnlinePstnUsage already defined"
} else {
    Write-Host "   Setting CsOnlinePstnUsage"
    Set-CsOnlinePstnUsage -Identity Global -Usage @{Add="$subdomain"}
    Start-Sleep -Seconds 2
}
Write-Host "   Done"


### Step 3: VOICE ROUTE

Write-Host "3. CsOnlineVoiceRoute rule"
Try
{
    if ((Get-CsOnlineVoiceRoute -Identity $subdomain -ErrorAction 'silentlycontinue') -eq $null)
    {
        Write-Host "   Creating CsOnlineVoiceRoute"
        New-CsOnlineVoiceRoute -Identity $subdomain  -NumberPattern ".*" -OnlinePstnGatewayList $subdomain -Priority 1 -OnlinePstnUsages $subdomain
        Start-Sleep -Seconds 2
    } else {
        if (!((Get-CsOnlineVoiceRoute -Identity $subdomain -ErrorAction 'silentlycontinue').OnlinePstnUsages -contains "$subdomain"))
        {
            Write-Host "   Setting CsOnlineVoiceRoute"
            Set-CsOnlineVoiceRoute -Identity $subdomain  -NumberPattern ".*" -OnlinePstnGatewayList $subdomain -Priority 1 -OnlinePstnUsages $subdomain
            Start-Sleep -Seconds 2
        }
        else {
            Write-Host "   CsOnlineVoiceRoute already created and configured"
        }
    }
} Catch {
    Write-Host "   Error processing CsOnlineVoiceRoute" -ForegroundColor Yellow
}
Write-Host "   Done"


### Step 4: VOICE ROUTING POLICY

Write-Host "4. CsOnlineVoiceRoutingPolicy rule" 
Try
{

    if ((Get-CsOnlineVoiceRoutingPolicy -Identity $subdomain -ErrorAction 'silentlycontinue') -eq $null)
    {
        Write-Host "   Creating CsOnlineVoiceRoutingPolicy"
        New-CsOnlineVoiceRoutingPolicy $subdomain -OnlinePstnUsages $subdomain
        Start-Sleep -Seconds 2
    } else {
        if (!((Get-CsOnlineVoiceRoutingPolicy -Identity $subdomain -ErrorAction 'silentlycontinue').OnlinePstnUsages -contains "$subdomain"))
        {
            Write-Host "   Setting CsOnlineVoiceRoutingPolicy"
            Set-CsOnlineVoiceRoutingPolicy $subdomain -OnlinePstnUsages $subdomain
            Start-Sleep -Seconds 2
        }
        else {
            Write-Host "   CsOnlineVoiceRoutingPolicy already created and configured"
        }
    }
} Catch {
    Write-Host "   Error processing CsOnlineVoiceRoutingPolicy" -ForegroundColor Yellow
}
Write-Host "   Done"


# Timer before running the report
Start-Sleep -Seconds 2


#############################################################################
#   Execution report to confirm everything has been properly set (or not)   #
#############################################################################
Write-Host "Execution report:"

if ((Get-CsOnlinePstnUsage -Filter Global).Usage -contains "$subdomain") {
    Write-Host " > CsOnlinePstnUsage... OK" -ForegroundColor Green
} else {
    Write-Host " > CsOnlinePstnUsage... NOK" -ForegroundColor Yellow
}


$route_check = (Get-CsOnlineVoiceRoute -Identity $subdomain -ErrorAction 'silentlycontinue').OnlinePstnUsages -contains "$subdomain"
if ($route_check) {
    Write-Host " > CsOnlineVoiceRoute... OK" -ForegroundColor Green
} else {
    Write-Host " > CsOnlineVoiceRoute... NOK" -ForegroundColor Yellow
}

$routing_policy_check = (Get-CsOnlineVoiceRoutingPolicy -Identity $subdomain -ErrorAction 'silentlycontinue').OnlinePstnUsages -contains "$subdomain"
if ($routing_policy_check) {
    Write-Host " > CsOnlineVoiceRoutingPolicy... OK" -ForegroundColor Green
} else {
    Write-Host " > CsOnlineVoiceRoutingPolicy... NOK" -ForegroundColor Yellow
}

Disconnect-MicrosoftTeams
Exit
