Write-Host "Script resource.ps1 started..."

$Version = @{
 Path = ".\resource.ps1"
 Version = "2.0"
 Author = "Sewan"
 Description = "Script for resource provisioning"
 }
Update-ScriptFileInfo @Version -Force -ErrorAction SilentlyContinue
Test-ScriptFileInfo -Path resource.ps1 -ErrorAction SilentlyContinue

#Checking the Teams PS Module
Try {
    $TeamsModuleVersion = Get-InstalledModule -Name MicrosoftTeams -RequiredVersion 2.3.1 -ErrorAction Stop
    Write-Host "Teams PS Module OK"
    } Catch { 
        Write-Warning "Your installed Teams PS module is not compatible with the script. Please install the validated version using the following command:" 
        Write-Host "Install-Module -Name MicrosoftTeams -RequiredVersion 2.3.1 -Force" -ForegroundColor Green
        Break
        }

#Start-Service winrm
Import-Module MicrosoftTeams

Function PrintResourceParameters($email) {
    Get-CsOnlineUser $email | Select-Object OnpremPhoneNumber,*Voice*,UserPrincipalName,RegistrarPool
}

Function ProvisionResourceAttribute ($email, $number, $subdomain) {
    
    # Resource's direct routing properties
    $CsUserProp = @{
        Identity               = $email
        OnpremPhoneNumber      = $number
									  
    }

	   
    If ((Get-CsOnlineUser $CsUserProp.Identity).TeamsUpgradeEffectiveMode -ne "TeamsOnly") {
        Try {
            Grant-CsTeamsUpgradePolicy -Identity $CsUserProp.Identity -PolicyName UpgradeToTeams -ErrorAction Stop
            Write-Host ">>> Resource set to TeamsOnly coexistence mode"
        } Catch { Write-Warning ">>> Failed to set Resource to TeamsOnly coexistence mode. Continuing to next resource." ; Continue }
    }

    Start-Sleep -Seconds 1

					
    Try {
        Set-CsOnlineApplicationInstance -identity $CsUserProp.Identity -OnpremPhoneNumber $CsUserProp.OnpremPhoneNumber -ErrorAction Stop
        Write-Host ">>> Resource parameters set (onprem line URI)"
    } Catch { Write-Warning ">>> Failed to assign Resource parameters" }

    Start-Sleep -Seconds 1

		 
    Try {
        Grant-CsOnlineVoiceRoutingPolicy -Identity $CsUserProp.Identity -PolicyName "Tag:$subdomain" -ErrorAction Stop
        Write-Host ">>> Resource has been granted routing policy"
    } Catch { Write-Warning ">>> Failed to grant routing policy to Resource" }

    Start-Sleep -Seconds 1

   if ($add_numdir -eq 0)
   {
   # Add ressource number to Azure AD
    Try {
        Set-AzureADUser -ObjectId $CsUserProp.Identity -TelephoneNumber $CsUserProp.OnpremPhoneNumber -ErrorAction Stop
        Write-Host ">>> Ressource phone number has been added to directory"
    } Catch { Write-Host ">>> Failed to add phone number to directory" -ForegroundColor Yellow }

    Start-Sleep -Seconds 1 
	}

}

# Prompt connection method
$title    = 'Authentication'
$question = "Are you using MFA ?"
$choices  = '&Yes', '&No'
$conn_method = $Host.UI.PromptForChoice($title, $question, $choices, 1)
							  																																	
# Prompt add phone number to Azure AD
$title    = 'Azure AD ressource phone number'
$question = "Do you want to add ressource's phone number to Azure AD ?"
$choices  = '&Yes', '&No'
$add_numdir = $Host.UI.PromptForChoice($title, $question, $choices, 1)

if ($add_numdir -eq 0)
{
   # Checking the AzureAD PS Module
   Try {
        $AzureModuleVersion = Get-InstalledModule -Name AzureAD -RequiredVersion 2.0.2.135 -ErrorAction Stop
        Write-Host "AzureAD PS module OK"
    } Catch { 
        Write-Warning "Your installed AzureAD PS module is not compatible with the script. Please install the validated version using the following command:" 
        Write-Host "Install-Module -Name AzureAD -RequiredVersion 2.0.2.135 -Force" -ForegroundColor Green
        Break
    }
}

if ($conn_method -eq 1)
{
    if (Test-Path ./global_vars.ps1 -PathType Leaf)
    {
        Write-Host "Global variables file found"
        . ./global_vars.ps1
        $passwd = convertto-securestring -AsPlainText -Force -String $password
        $subdomain = $activationuser.Split("{@}").Get(1)
    }
    else 
    {
        Write-Warning "Global variables global_vars.ps1 not found"
        Write-Host "Please fill in the following fields:"

        $adminaccount = Read-Host -Prompt "Admin login"
        $passwd = Read-Host -Prompt "Admin password" -AsSecureString
        $subdomain = Read-Host -Prompt "Teams softphone domain (ex: XXXXXX.teamssoftphone.com)"
    }
}
else
{
[void][System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic')
$subdomain = [Microsoft.VisualBasic.Interaction]::InputBox("Your Teams softphone domain (example: 123456.teamssoftphone.com)", "Please fill in")
}

$use_csv = $False
if (! (Test-Path ./teamsresources.csv -PathType Leaf))
{
    Write-Host "teamsresources.csv missing, please fill in the following fields:"
    $PhoneNumber = Read-Host -Prompt "Resource PhoneNumber (E.164)"
    $UserUPN = Read-Host -Prompt "Resource UPN (email)"

} 
else { $use_csv = $True }


			   
if ($conn_method -eq 1)
{
# Connection part (not using MFA)
$cred = New-Object System.Management.Automation.PSCredential ($adminaccount, $passwd)
$sfboSession = Connect-MicrosoftTeams -Credential $cred

Write-Host "Connected to Teams"
Start-Sleep -Milliseconds 500
    
    if ($add_numdir -eq 0)
    {
    $sfboSessionazure = Connect-AzureAD -Credential $cred
    Write-Host "Connected to Azure AD"
    Start-Sleep -Milliseconds 500
    }    
}
else {

# Connection part (using MFA)
$sfboSession = Connect-MicrosoftTeams
Write-Host "Connected to Teams"
Start-Sleep -Milliseconds 500
    
    if ($add_numdir -eq 0)
    {
    $sfboSessionazure = Connect-AzureAD
    Write-Host "Connected to Azure AD"
    Start-Sleep -Milliseconds 500
    }
}

if ($use_csv)
{
    ForEach ($u in (Import-Csv ./teamsresources.csv)) 
    {
        $UserUPN = $u.UserUPN
        $PhoneNumber = $u.PhoneNumber

        Write-host "> $UserUPN"
        Write-host "Before:"
        PrintResourceParameters $UserUPN
        ProvisionResourceAttribute $UserUPN $PhoneNumber $subdomain
        Write-host "After:"
        PrintResourceParameters $UserUPN
        Write-host "> $UserUPN done" -ForegroundColor Green
    }
}
else 
{
    Write-host "> $UserUPN"
    Write-host "Before:"
    PrintResourceParameters $UserUPN
    ProvisionResourceAttribute $UserUPN $PhoneNumber $subdomain
    Write-host "After:"
    PrintResourceParameters $UserUPN
    Write-host "> $UserUPN done" -ForegroundColor Green
}

if ($add_numdir -eq 0) {
Disconnect-AzureAD 
}
Disconnect-MicrosoftTeams
Exit
